/*
 * Converts ausearch output into quoted, easily parsed name-value pairs.
 * John D. Ramsdell--The MITRE Corporation
 *
 * Copyright (C) 2006 The MITRE Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* For each audit event, this program generates a sequence of records.
   Each record is consists of a sequence of name-value pairs. */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include "scanner.h"
#include "emit.h"

/* Emit a record.  Not used for AVC records. */
static void
emit_record(void)
{
  int not_first = 0;
  int token;
  do {				/* For each name-value pair */
    if (not_first)
      emit_field_separator();
    else
      not_first = 1;
    emit_item(getyytext());	/* print name, */
    emit_name_value_separator();
    token = yylex();
    if (token == EQ) {
      token = yylex();
      if (token != ITEM) {
	yyerror("Missing value after equal sign");
	exit(1);
      }
      emit_item(getyytext());	/* and then its value. */
      token = yylex();
    }
  } while (token == ITEM);
  emit_record_end();
  if (token != NL) {
    yyerror("Expecting the end of a record");
    exit(1);
  }
}

/* Emit a line of text that begins with "type=AVC". */
static void
emit_avc(void)
{
  emit_item("type");
  emit_name_value_separator();
  emit_item("AVC");
  emit_field_separator();
  emit_item("msg");
  emit_name_value_separator();
  emit_item(getyytext());
  emit_record_end();
  if (yylex() != NL) {
    yyerror("Expecting a new line");
    exit(1);
  }
}

/* Starting point for parsing audit output syntax, and converting it
   into easily parsed, quoted name-value pair syntax. */
static int
filter(void)
{
  int token = yylex();		/* Skip leading separator. */
  if (token != SEP) {
    yyerror("Expecting a separator");
    return 1;
  }
  token = yylex();
  if (token != NL) {
    yyerror("Expecting a new line");
    return 1;
  }
  for (;;) {			/* For each group of related records */
    emit_start_event();
    token = yylex();
    for (;;) {
      if (token == AVC)
	emit_avc();
      else if (token == ITEM)
	emit_record();
      else {
	yyerror("Parse error");
	return 1;
      }
      token = yylex();
      if (token == DONE) {	/* No more input. */
	return 0;
      }
      else if (token == SEP) {	/* Finish this group, and go on. */
	token = yylex();
	if (token != NL) {
	  yyerror("Expecting a new line");
	  return 1;
	}
	break;
      }
    }
  }
}

/* Generic filtering main and usage routines. */

static void
version(void)
{
  fprintf(stderr, "%s\n", PACKAGE_STRING);
}

static const char usage_format[] =
  "Usage: %s [OPTIONS] [FILE]\n"
  "Convert audit output in FILE into easily parsed, quoted name-value pairs.\n"
  "\n"
  "  -o FILE, --output=FILE     output to FILE (default is standard output)\n"
  "  -t,      --tsv             emit tab separated values\n"
  "  -v,      --version         print version information and exit\n"
  "  -h,      --help            print this message and exit\n"
  "\n"
  "With no FILE, or when FILE is -, read standard input.\n";

static void
usage(const char *prog)
{
  fprintf(stderr, usage_format, prog);
}

int
main(int argc, char **argv)
{

  char *output = NULL;
  set_tsv_mode(0);

  for (;;) {
    static struct option long_options[] = {
      {"output", 1, 0, 'o'},
      {"tabs", 1, 0, 't'},
      {"version", 0, 0, 'v'},
      {"help", 0, 0, 'h'},
      {NULL, 0, 0, 0}
    };
    int c = getopt_long(argc, argv, "o:tvh", long_options, NULL);
    if (c == -1)
      break;
    switch (c) {
    case 'o':
      output = optarg;
      break;
    case 't':
      set_tsv_mode(1);
      break;
    case 'v':
      version();
      return 0;
    case 'h':
      usage(argv[0]);
      return 0;
    default:
      usage(argv[0]);
      return 1;
    }
  }

  switch (argc - optind) {
  case 0:			/* Use stdin */
    break;
  case 1:
    if (strcmp("-", argv[optind]) && !freopen(argv[optind], "r", stdin)) {
      perror(argv[optind]);
      return 1;
    }
    setfile(argv[optind]);
    break;
  default:
    fprintf(stderr, "Bad arg count\n");
    usage(argv[0]);
    return 1;
  }

  if (output && !freopen(output, "w", stdout)) {
    perror(output);
    return 1;
  }

  return filter();
}
