/*
 * Generates ASCII versions of audit records that are easily parsed.
 * John D. Ramsdell--The MITRE Corporation
 *
 * Copyright (C) 2006 The MITRE Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <ctype.h>
#include "emit.h"

/* See the header file for exported function documentation. */

static int tsv_mode = 0;

void
set_tsv_mode(int flag)
{
  tsv_mode = flag;
}

void
emit_start_event(void)
{
  printf("---\n");
}

void
emit_record_end(void)
{
  putchar('\n');
}

void
emit_field_separator(void)
{
  if (tsv_mode)
    putchar('\t');
  else
    putchar(' ');
}

void
emit_name_value_separator(void)
{
  if (tsv_mode)
    putchar('\t');
  else
    putchar('=');
}

/* Generates the contents of a C string literal. */

static void
emit_c_string(const char *bytes)
{
  for (;;) {
    int ch = *bytes++;
    if (!ch)
      return;
    else if (isalnum(ch))
      putchar(ch);
    else switch (ch) {
    case ' ': case '!': case '#': case '%': case '^': case '&':
    case '*': case '(': case '_': case ')': case '-': case '+':
    case '=': case '~': case '[': case ']': case '\'': case '|':
    case ';': case ':': case '{': case '}': case ',': case '.':
    case '<': case '>': case '/': case '?':
      putchar(ch);
      break;
    case '\a':
      putchar('\\');
      putchar('a');
      break;
    case '\b':
      putchar('\\');
      putchar('b');
      break;
    case '\f':
      putchar('\\');
      putchar('f');
      break;
    case '\n':
      putchar('\\');
      putchar('n');
      break;
    case '\r':
      putchar('\\');
      putchar('r');
      break;
    case '\t':
      putchar('\\');
      putchar('t');
      break;
    case '\v':
      putchar('\\');
      putchar('v');
      break;
    case '\\':
      putchar('\\');
      putchar('\\');
      break;
    case '\"':
      putchar('\\');
      putchar('"');
      break;
    default:
      putchar('\\');
      putchar(((ch & (3 << 6)) >> 6) + '0');
      putchar(((ch & (7 << 3)) >> 3) + '0');
      putchar((ch & 7) + '0');
    }
  }
}

/* See if the string requires no escapes, and does not contain a space
   or an equal sign. */

static int
needs_no_escapes(const char *bytes)
{
  for (;;) {
    int ch = *bytes++;
    if (!ch)
      return 1;
    else if (isalnum(ch))
      continue;
    else switch (ch) { /* Note space and the equal sign are missing */
    case '!': case '#': case '%': case '^': case '&':
    case '*': case '(': case '_': case ')': case '-': case '+':
    case '~': case '[': case ']': case '\'': case '|':
    case ';': case ':': case '{': case '}': case ',': case '.':
    case '<': case '>': case '/': case '?':
      continue;
    default:
      return 0;
    }
  }
}

void
emit_item(const char *bytes)
{
  if (tsv_mode)
    emit_c_string(bytes);
  else if (needs_no_escapes(bytes))
    printf("%s", bytes);
  else {
    putchar('"');
    emit_c_string(bytes);
    putchar('"');
  }
}
