""" Decisions that have to be made.
1. Is this software to be part of the kernel,
in the system layer, in the admin layer, a service, or an application.

2. Is this to be basic, module, or inactive.  This information
is entered in modules.conf

3. Is policy to be creating a monolithic or modular policy.  Monolithic
is all compiled together and doesn't have incremental benefits of using modules.

There are several new constructs that may be placed in .te file
These are policy_module(<app_name>), optional_policy(), tunable_policy(), ifdef()

Cross boundary issue.  Key is finding the right interface call.
Say you know you need to read from etc. Probably you want
files_read_etc_files(jabberd_t).  However there may be several interface calls
that contain the right "allow" you are looking for.  Which one satisfies the
constraints. For now we are picking the one that allows fewest privilege.

Future work:

1. We can consider making a global decision on interface call. Globally you might need all the items in an interface call
anyway. So there is a global issue: given need from {allows} how can you match as many
as possible by selecting the right interface call?

2. Interface calls can lead to confusion.  init_system_domain also sets up roles. So if
you don't have an application that starts up at init time, you are going to have to
add a role declaration into your policy - otherwise, the context will be wrong and
you will not have permission to run the application.

3. may need to update unconfined module - e.g., iptables_domtrans(unconfined_t)

4. Can we make generation more incremental - generate files, run make_load, fix, and iterate
or come back later and make one small change without having to redo the entire generation.
Of course users can change .te, but there are situations where this is best done as a
transformation.

5. How to get from psl to capabilities? e.g., administering network
functionality -> net_admin capability"""

from util import *
from dutil import if_dict,interface_needs
import config
from psl_parsing import nodes

using_types = []

class GenIF:
    """For generating the .if file"""
    def get_strm(self):
        self.strm = open(config.data_directory + '/results/' \
                         + get_current_program().name + \
                         '.if', 'w')
        return self.strm
    
    def do_it(self, visitor):
        self.do_it_all(self.strm, visitor)
        
    def do_it_all(self, strm, visitor, format, ty_declared):
        #strm.write(format.start)
        #ifs should come from provides + any uncovered allows
        self.ty_declared = ty_declared
        for item in interface_needs:
            print item
            for x in item.evidence_for:
                print "    " + str(x)
        ifs = nodes[get_current_program().name].provides
        linefeed = format.linefeed
        lt = '<'
        gt = '>'
        if linefeed != '':
            lt = '&lt;'
            gt = '&gt;'
        strm.write(format.start + '## ' + lt + "summary" + gt +\
                   get_current_program().name \
                   + lt + "/summary" + gt + format.end + linefeed + '\n')
        #print ty_declared
        for interf in ifs:
            requires = if_dict[interf].required_types
            good = True
            for r in requires:
                matches = [x for x in ty_declared if contains_str(x, r)]
                if len(matches) == 0:
                    print "$$$ Cannot generate " + interf + " interface, because " + \
                          r + " not in declared types"
                    good = False
                    break
            if good:
                 self.do_interface(strm, if_dict[interf], visitor, format)
        strm.write(format.ending)
            
    def do_interface(self, strm, info, visitor, format):
        visitor.format_request = format
        visitor.gen_interface_intro(strm, info)
        visitor.gen_summary(strm, info)
        visitor.gen_params(strm, info)
        visitor.gen_name(strm, info)
        visitor.gen_requires(strm, info, self.ty_declared)
        visitor.gen_macros(strm, info)
        visitor.gen_allows(strm, info)
        visitor.gen_end(strm)



class IFVisitor:
    """ Subclasses will handle language-specific (e.g., M4) generation""" 
    def __init__(self, prgm_name, name):
        self.name = name
        self.prgm_name = prgm_name
        self.generic_to_local_map = {}

    def gen_interface_intro(self, strm, ifobj):
        linefeed = self.format_request.linefeed
        strm.write(linefeed + linefeed + "\n########################### \n")

    def gen_summary(self, strm, ifobj):
        self.gen_summary_aux(strm, ifobj.brief_desc)

    def gen_summary_aux(self, strm, desc):
        linefeed = self.format_request.linefeed
        lt = '<'
        gt = '>'
        if linefeed != '':
            lt = '&lt;'
            gt = '&gt;'
        strm.write(linefeed + "## " + lt + "summary" + gt + " \n" + linefeed + "## " \
                   + desc + " \n" + linefeed + "## " + lt + "/summary" + gt + "\n")
        

    def gen_params(self, strm, ifobj):
        linefeed = self.format_request.linefeed
        lt = '<'
        gt = '>'
        if linefeed != '':
            lt = '&lt;'
            gt = '&gt;'
        if len(ifobj.params) == 0: #no params
            self.do_params(strm, linefeed, lt, gt, "domain")
        else:
            for p in ifobj.params:
                self.do_params(strm, linefeed, lt, gt, p)

    def do_params(self, strm, linefeed, lt, gt, p):
        strm.write(linefeed + '## ' + lt + 'param name = "' + p + '" ' + \
                       gt + ' \n')
        self.gen_summary_aux(strm, "Allow access")
        strm.write(linefeed + '## ' + lt + '/param' + gt + ' \n' + \
                       linefeed + '#\n'\
                       + linefeed)

class M4Writer (IFVisitor):
    def __init__(self, prgm_name, name = 'M4 Generator'):
        IFVisitor.__init__(self, prgm_name, name)
        self.target_lang = 'M4'
        
    def gen_name (self, strm, ifobj):
        linefeed = self.format_request.linefeed
        focus = self.format_request.focus
        end_focus = self.format_request.end_focus
        strm.write(linefeed + linefeed + "\n\ninterface(`" + focus + self.prgm_name \
               + "_" + ifobj.name + end_focus + "',` \n" + linefeed)
        
    def gen_requires (self, strm, ifobj, ty_declared):
        linefeed = self.format_request.linefeed
        strm.write('     gen_require(` \n' + linefeed)
        for ty in ifobj.required_types:
            matches = if_matches(ty, ty_declared)
            self.generic_to_local_map[ty] = matches
            if matches != '':
                strm.write('         type ' + matches + '; \n' + linefeed)
        strm.write("     ') \n" + linefeed)

    def gen_macros (self, strm, ifobj):
        linefeed = self.format_request.linefeed
        for a in ifobj.macros: # each macro is of form [name [arg1, ...]]
            name = a[0]
            args = a[1]
            to_write = '      ' + name + '('
            for arg in args:
                if arg == '$1':
                    to_write = to_write + '$1, '
                else:
                    to_write = to_write + self.generic_to_local_map[arg] + ', '
            strm.write(to_write[:-2] + ') \n\n')
                    

    def gen_allows (self, strm, ifobj):
        linefeed = self.format_request.linefeed
        for a in ifobj.allows:
            parts = a.split(':')
            if parts[0] in self.generic_to_local_map.keys():
                new_text = self.generic_to_local_map[parts[0]] + ':' + parts[1]
                strm.write('     allow $1 ' + new_text + '; \n' \
                       + linefeed)
        for a in ifobj.reverse_allows:
            text = a[0]
            if text in self.generic_to_local_map.keys():
                new_text = self.generic_to_local_map[text]
                strm.write('     allow ' + new_text + ' $1:' + \
                a[1] + '; \n' + linefeed)
        
    def gen_end (self, strm):
        strm.write("')")

def if_matches (x, ty_declared):
    if x == 'data_t':
        to_return = []
        for y in ['dir', 'data']:
            to_return = to_return + [ty for ty in ty_declared \
                                     if contains_str(ty, y)]
    else:
        to_return = [ty for ty in ty_declared if contains_str(ty, x)]
    min_length = 1000
    minstr = ""
    for item in to_return:
        l = len(item)
        if l < min_length:
            min_length = l
            minstr = item
    return minstr
