#! /usr/bin/env python

# Converts ausearch system call output, formatted as tab separated
# values, into a format that can be consumed by the trackfd program.
# The program auinterp converts ausearch system call output into the
# tab separated values format this program expects.

import sys, getopt

def main():
    output_file = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], "ho:", ["help", "output="])
    except getopt.GetoptError:          # bad options
        usage()                         
        sys.exit(2)
    for o, a in opts:
        if o in ("-h", "--help"):       # print help information
            usage()
            sys.exit(0)
        elif o in ("-o", "--output"):   # record output file name
            output_file = a
    if len(args) > 1:                # too many command line arguments
        usage()
        sys.exit(2)
    elif len(args) == 1:
        # open input file before the output file.
        sys.stdin.close();
        sys.stdin = open(args[0], 'r')
    if output_file:
        try:
            sys.stdout.close()
            sys.stdout = open(output_file, 'w')
        except IOError:
            sys.stderr.write("cannot open " + output_file + " for writing")
            sys.exit(1)
    filter()

def usage():
    print "Usage: " + sys.argv[0] + " [-h] [-o OUTPUT] [INPUT]"

def filter():
    seq = None      # A sequence of tables representing an audit event
    lineno = 0
    seqno = 0
    while True:
        line = sys.stdin.readline()
        if not line:
	    if seq:
		consume(seq, seqno)
            return
        lineno = lineno + 1
	if line == "---\n":
	    if seq:
		consume(seq, seqno)
            seq = []
            seqno = lineno
	    continue
        record = line.strip().split("\t")
        nf = len(record)                # number of fields
	if nf % 2 != 0:
	    sys.stderr.write("Bad field count on line " + str(lineno) + "\n")
	    sys.exit(1)
	tab = {}
        for i in range(0, nf, 2):
	    tab[record[i]] = record[i + 1]
        seq.append(tab)

def consume(seq, seqno):
    global pat                          # pat contains the syscall 
    if has_type("SYSCALL", seq):        # patterns used to drive output
	pattern = None
        syscall = None
	try:
	    syscall = search("syscall", seq)
	    pattern = pat[syscall]
	except KeyError:                # When event not a syscall, return
	    return
        sys.stdout.write(search("pid", seq))
        sys.stdout.write("\t" + syscall)
	for template in pattern:
	    if isinstance(template, str):
		p = search(template, seq)
		sys.stdout.write("\t" + p)
	    elif isinstance(template, int): # Get info from PATH record
		p = path_name(template, seq)
		sys.stdout.write("\t" + p)
	    else:
		sys.stdout.write("\t!!!")
	sys.stdout.write("\t" + search("exit", seq))
        sys.stdout.write("\t" + search("subj", seq))
        sys.stdout.write("\n")

def has_type(typ, seq):                 # See if seq has a record
    for tab in seq:                     # of the given type
	try:
	    if tab["type"] == typ:
		return True
	except KeyError:
	    pass
    return False

def search(key, seq):                   # Return some value associated
    n = len(seq) - 1                    # with the given key,
    if n < 0:                           # or raise KerError
	raise KeyError
    for i in range(n):
	try:
	    return seq[i][key]
	except KeyError:
	    pass
    return seq[n][key]

def path_name(item, seq):               # Return the name and the 
    n = len(seq) - 1                    # security context of a 
    if n < 0:                           # PATH record that matches
	raise KeyError                  # the given item number.
    for i in range(n):
	try:
	    return path_name_tab(item, seq[i])
	except KeyError:
	    pass
    return path_name_tab(item, seq[n])

def path_name_tab(item, tab):
    if tab["type"] != "PATH" or tab["item"] != str(item):
	raise KeyError
    return tab["name"] + "\t" + tab["obj"]

# pat contains the syscall patterns used to drive output.  Each key
# maps a system call to the template used for the given system call.
# A template is a list of format indicators.  A string represents the
# key used to look up a value, which is used for the output.  A number
# represents the item number of a PATH type, and causes the name field
# and the obj field to be inserted into the output.  Any other type of
# object causes the string "!!!" to be substituted into the output.

pat = {
    "execve":[0, "a1", "a2"],
    "clone":["a0", "a1", "a2"],
    "close":["a0", None],
    "dup":["a0", None],
    "dup2":["a0", None, "a1", None],
    "fcntl64":["a0", None, "a1", "a2"],
    "open":[0, "a1"],
    "pipe":["a0"],
    "read":["a0", None, "a1", "a2"],
    "write":["a0", None, "a1", "a2"],
}

if __name__ == "__main__":
    main()
